@@ -1,12 +1,25 @@
+"""anonymous registration form and views
+
+:organization: Logilab / SecondWeb
+:copyright: 2009-2010 LOGILAB S.A. (Paris, FRANCE), license is LGPL v2.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
+"""
+__docformat__ = "restructuredtext en"
+
 from logilab.common.decorators import clear_cache
 
+from yams.schema import role_name
+
 from cubicweb import mail, crypto
 from cubicweb.view import StartupView
-from cubicweb.web import Redirect, ValidationError
+from cubicweb.web import Redirect, ValidationError, ProcessFormError
 from cubicweb.web import controller, form, captcha
 from cubicweb.web import formwidgets as fw, formfields as ff
 from cubicweb.web.views import forms, basecomponents, urlrewrite
 
+def qname(attr):
+    return role_name(attr, 'subject')
 
 
 class RegistrationForm(forms.FieldsForm):
@@ -18,18 +31,23 @@ class RegistrationForm(forms.FieldsForm):
     def action(self):
         return self._cw.build_url(u'registration_sendmail')
 
-    login = ff.StringField(widget=fw.TextInput(), required=True)
-    upassword = ff.StringField(widget=fw.PasswordInput(), required=True)
-    use_email = ff.StringField(widget=fw.TextInput(), required=True)
-    firstname = ff.StringField(widget=fw.TextInput())
-    surname = ff.StringField(widget=fw.TextInput())
+    # properly name fields according to validation errors that may be raised by
+    # Repository.register_user
+    login = ff.StringField(widget=fw.TextInput(), role='subject',
+                           # we don't want to see 'authenticate'
+                           label=_('i18n_register_login'),
+                           required=True)
+    upassword = ff.StringField(widget=fw.PasswordInput(), role='subject',
+                               required=True)
+    address = ff.StringField(widget=fw.TextInput(), role='subject',
+                               required=True)
+    firstname = ff.StringField(widget=fw.TextInput(), role='subject')
+    surname = ff.StringField(widget=fw.TextInput(), role='subject')
     captcha = ff.StringField(widget=captcha.CaptchaWidget(), required=True,
                              label=_('captcha'),
                              help=_('please copy the letters from the image'))
 
 
-# XXX move captcha validation to a CaptchaField
-
 class RegistrationFormView(form.FormViewMixIn, StartupView):
     __regid__ = 'registration'
 
@@ -37,11 +55,10 @@ class RegistrationFormView(form.FormViewMixIn, StartupView):
         form = self._cw.vreg['forms'].select('registration', self._cw)
         self.w(form.render(display_progress_div=False))
 
-
 class RegistrationSendMailController(controller.Controller):
     __regid__ = 'registration_sendmail'
     content = _(u'''
-Hello %(firstname)s %(surname)s,
+Hello %(firstname-subject)s %(surname-subject)s,
 
 thanks for registering on %(base_url)s.
 
@@ -54,7 +71,7 @@ See you soon on %(base_url)s !
 
     def publish(self, rset=None):
         data = self.checked_data()
-        recipient = data['use_email']
+        recipient = data[qname('address')]
         msg = self.build_email(recipient, data)
         self._cw.vreg.config.sendmails([(msg, (recipient,))])
         raise Redirect(self.success_redirect_url())
@@ -63,37 +80,36 @@ See you soon on %(base_url)s !
         '''only basic data check here (required attributes and password
         confirmation check)
         '''
-        fieldsform = self._cw.vreg['forms'].select('registration', self._cw)
+        form = self._cw.vreg['forms'].select('registration', self._cw)
+        form.formvalues = {} # init fields value cache
         data = {}
         errors = {}
-        for field in fieldsform._fields_:
-            value = self._cw.form.get(field.name, u'').strip()
-            if not value:
-                if field.required:
-                    errors[field.name] = self._cw._('required attribute')
-            data[field.name] = value
-        if data['upassword'] != self._cw.form.get('upassword-confirm'):
-            errors['upassword'] = _('passwords are different')
-        captcha = self._cw.get_session_data('captcha', None, pop=True)
-        if captcha is None:
-            errors[None] = self._cw._('unable to check captcha, please try again')
-        elif data['captcha'].lower() != captcha.lower():
-            errors['captcha'] = self._cw._('incorrect captcha value')
+        for field in form.fields:
+            try:
+                for field, value in field.process_posted(form):
+                    if value is not None:
+                        data[field.role_name()] = value
+            except ProcessFormError, exc:
+                errors[field.role_name()] = unicode(exc)
         if errors:
             raise ValidationError(None, errors)
         return data
 
     def build_email(self, recipient, data):
-        if not data.get('firstname') or data.get('surtname'):
-            data['firstname'] = data['login']
+        activationurl = self.activation_url(data) # build url before modifying data
+        data.setdefault(qname('firstname'), '')
+        data.setdefault(qname('surname'), '')
+        if not (data.get(qname('firstname')) or data.get(qname('surname'))):
+            data[qname('firstname')] = data[qname('login')]
         data.update({'base_url': self._cw.vreg.config['base-url'],
-                    'url': self.activation_url(data)})
+                     'url': activationurl})
         content = self._cw._(self.content) % data
         subject = self._cw._(self.subject) % data
         return mail.format_mail({}, [recipient], content=content,
                                 subject=subject, config=self._cw.vreg.config)
 
     def activation_url(self, data):
+        data.pop(qname('upassword') + '-confirm', None)
         key = crypto.encrypt(data, self._cw.vreg.config['registration-cypher-seed'])
         return self._cw.build_url('registration_confirm', key=key)
 
@@ -111,20 +127,20 @@ class RegistrationConfirmController(controller.Controller):
         try:
             data = crypto.decrypt(req.form['key'],
                                   req.vreg.config['registration-cypher-seed'])
-            self.debug('registration data: %s', data)
+            login = data[qname('login')]
+            password = data.pop(qname('upassword'))
         except:
             msg = req._(u'Invalid registration data. Please try registering again.')
             raise Redirect(req.build_url(u'register', __message=msg))
-        login, password = data.pop('login'), data.pop('upassword')
-        try:
-            self.appli.repo.register_user(login, password, email=data['use_email'],
-                                          firstname=data['firstname'],
-                                          surname=data['surname'])
-        except ValidationError, e:
-            raise Redirect(self.failure_redirect_url(data, e.errors))
+        req.form = data # hijack for proper validation error handling
+        self.appli.repo.register_user(login, password,
+                                      email=data.get(qname('address')),
+                                      firstname=data.get(qname('firstname')),
+                                      surname=data.get(qname('surname')))
+        # force new authentication (anon until there)
+        clear_cache(req, 'get_authorization')
         req.form['__login'] = login
         req.form['__password'] = password
-        clear_cache(req, 'get_authorization') # force new authentication (anon until there)
         if req.cnx:
             req.cnx.close()
         req.cnx = None
@@ -135,17 +151,9 @@ class RegistrationConfirmController(controller.Controller):
         assert req.user.login == login
         raise Redirect(self.success_redirect_url(data))
 
-    def failure_redirect_url(self, data, errors):
-        # be sure not to get a password in your redirect url
-        data.pop('upassword', None)
-        data.pop('upassword-confirm', None)
-        data['__message'] = u'<br/>'.join(u'%s : %s' % (k,v)
-                                          for (k,v) in errors.iteritems())
-        return self._cw.build_url(u'register', **data)
-
     def success_redirect_url(self, data):
         msg = self._cw._(u'Congratulations, your registration is complete. '
-                         'Welcome %(firstname)s %(surname)s !')
+                         'Welcome %(firstname-subject)s %(surname-subject)s !')
         return self._cw.build_url('', __message=msg%data)
 
 
